require 'crusade/logger'
require 'crusade/apns/socket_connection'

module Crusade
  module APNS
    class PushNotification
      include Crusade::Logger

      attr_reader :configuration

      def initialize(configuration, connection = nil)
        self.configuration = configuration
        self.encoder = NotificationEncoder.new(configuration)
        self.connection = connection
      end

      def send notification
        return :invalid unless Array(notification.url_args).count == url_args_in_configuration
        
        info "sending notification #{notification.inspect}"
        connection.send encoder.encode notification
      end

      private

      attr_accessor :encoder
      attr_writer   :configuration, :connection

      def connection
        @connection ||= Crusade::APNS::SocketConnection.new(configuration)
      end

      def url_args_in_configuration
        configuration.url_format.scan("%@").count
      end
    end
  end
end