require 'json'
require 'pathname'
require 'digest'

module Crusade
  module APNS
    class ManifestGenerator
      def initialize directory
        self.directory = directory
      end

      def generate
        to_hash.to_json
      end

      private

      attr_accessor :directory

      def to_hash
        Dir.glob("#{directory}/**/*").inject({}) do |memo,file_path|
          if should_digest? file_path
            memo[relative_path(file_path)] = digest file_path
          end
          memo
        end
      end

      def relative_path absolute_path
        Pathname.new(absolute_path).relative_path_from(Pathname.new(directory))
      end

      def should_digest? absolute_path
        File.file? absolute_path
      end

      def digest file_path
        Digest::SHA1.hexdigest File.read(file_path)
      end
    end
  end
end