require 'openssl'
require 'base64'

module Crusade
  module APNS
    class SignatureGenerator
      FLAG = OpenSSL::PKCS7::DETACHED | OpenSSL::PKCS7::BINARY

      def initialize configuration
        self.configuration = configuration
      end

      def sign manifest
        dir = File.dirname manifest
        signature_file = File.join(dir, 'signature')

        File.open(signature_file, 'wb') do |file|
          file.write generate_signature manifest
        end
      end

      private

      attr_accessor :configuration

      def generate_signature manifest
        crt = certificate

        signature = OpenSSL::PKCS7::sign(crt.certificate, crt.key, File.read(manifest), [], FLAG  )
        signature = signature.to_s.split("\n")
        signature = signature[1..signature.length-2].join("\n")

        Base64.decode64 signature.to_s
      end

      def certificate
        crt = OpenSSL::PKCS12.new(File.read(configuration.certificate), certificate_password)
      end

      def certificate_password
        configuration.certificate_password
      end
    end
  end
end