require 'crusade/logger'
require 'crusade/apns/push_package/directory_structure_generator'
require 'crusade/apns/push_package/website_file_generator'
require 'crusade/apns/push_package/manifest_generator'
require 'crusade/apns/push_package/signature_generator'
require 'crusade/apns/push_package/zip_file_generator'

module Crusade
  module APNS
    class PushPackageGenerator
      include Crusade::Logger

      def initialize configuration
        self.configuration = configuration
      end

      def generate user_id
        info "generating push package for user #{user_id}"

        FileUtils.remove_entry_secure configuration.temp_dir
        FileUtils.mkdir configuration.temp_dir

        push_package = File.join(configuration.temp_dir, file_name(user_id))

        DirectoryStructureGenerator.new(configuration).generate

        website = WebsiteFileGenerator.new(user_id, configuration).generate
        user_token = user_token website
        File.open(File.join(configuration.temp_dir, 'website.json'), 'w') { |f| f.write website }

        manifest = Crusade::APNS::ManifestGenerator.new(configuration.temp_dir).generate
        File.open(File.join(configuration.temp_dir, 'manifest.json'), 'w') { |f| f.write manifest }

        Crusade::APNS::SignatureGenerator.new(configuration).sign File.join(configuration.temp_dir, 'manifest.json')

        Crusade::APNS::ZipFileGenerator.new(configuration, configuration.temp_dir, push_package).generate

        debug "push package generated #{push_package} - with user token #{user_token}"
        [ user_token, push_package ]
      end

      def clean(push_package)
        FileUtils.remove_entry_secure push_package
      rescue Errno::ENOENT => e
      end

      private

      attr_accessor :configuration

      def file_name(user_token)
        "#{Time.now.to_i}-#{user_token}_push_package.zip"
      end

      def user_token json
        JSON.parse(json)['authenticationToken']
      end
    end
  end
end